import json
import sys
from datetime import datetime
import tzlocal


class PrettyClass:
    useless_attrs = ["api_response", "agent"]

    def __str__(self):
        useful_attrs = ["{}: {}".format(key, val) for key, val in self.__dict__.items() if
                        key not in self.useless_attrs]

        return "{}(\n\t{}\n)".format(
            self.__class__.__name__,
            ",\n\t".join(useful_attrs)
        )

    def __repr__(self):
        return "\n" + indent(str(self))

    def to_tuple(self):
        return tuple([val for key, val in self.__dict__.items() if key not in self.useless_attrs])


class QuandooModel(PrettyClass):

    def __init__(self, data):
        self.api_response = data

    def get_api_response(self):
        return json.dumps(self.api_response, indent=2)


class QuandooDatetime(PrettyClass):
    TIME_RESOLUTION = 15  # minutes

    def __init__(self, *args, **kwargs):
        if type(args[0]) == datetime and len(args) == 1:
            self.datetime = args[0]
        else:
            self.datetime = datetime(*args, **kwargs)

        if not self.datetime.tzinfo:
            self.datetime = tzlocal.get_localzone().localize(self.datetime)

        self.__resolve_time()

    def __str__(self):
        useful_attrs = ["{}: {}".format(key, val) for key, val in self.__dict__.items() if
                        key not in self.useless_attrs] + ["{}: {}".format("q_datetime", self.get_qdt()),
                                                          "{}: {}".format("pretty_date", self.pretty_date())]

        return "{}(\n\t{}\n)".format(
            self.__class__.__name__,
            ",\n\t".join(useful_attrs)
        )

    def __eq__(self, other):
        return self.datetime == other.datetime

    def __lt__(self, other):
        return self.datetime > other.datetime

    @staticmethod
    def now():
        return QuandooDatetime(datetime.now())

    @staticmethod
    def parse_str_qdt(string):
        d = string.split("+")
        d[1] = d[1].replace(":", "")
        return QuandooDatetime(datetime.strptime("+".join(d), "%Y-%m-%dT%H:%M:%S%z"))

    @staticmethod
    def parse_pretty_qdt(string):
        return QuandooDatetime(datetime.strptime(string, "%I:%M %p, %a %d %B %Y"))

    def __resolve_time(self):
        minute = (self.datetime.minute // QuandooDatetime.TIME_RESOLUTION) * QuandooDatetime.TIME_RESOLUTION
        second = microsecond = 0

        self.datetime = self.datetime.replace(minute=minute, second=second, microsecond=microsecond)

    def get_qdt(self):
        return self.datetime.__str__().replace(' ', 'T')

    def get_urldt(self):
        return self.datetime.strftime("%Y-%m-%d %H:%M:%S")

    def pretty_date(self):
        if sys.platform.startswith("win"):
            token = "#"
        elif sys.platform.startswith("linux"):
            token = "-"
        else:
            token = ""
        return self.datetime.strftime("%{token}I:%M %p, %a %{token}d %B %Y".format(**{"token": token}))


def urljoin(*argv):
    return "/".join([str(arg) for arg in argv])


def indent(string, indent_amount=1):
    return "\n".join(["\t" * indent_amount + line for line in string.split("\n")])



import unittest

from quandoo.Customer import Customer
from . import agent


class MyTestCase(unittest.TestCase):
    customer_id = None

    def test_creation(self):
        customer = Customer(
            {
                'id': None,
                "firstName": 'test',
                "lastName": 'test',
                "email": 'test@mail.com',
                "phoneNumber": '+61411222333',
            },
            agent)
        print(merchant.create_reservation(customer, 2, start_qdt, extra_info=e).get_reservation())
        self.assertEqual(True, False)


if __name__ == '__main__':
    unittest.main()



import json

import requests

from quandoo.Error import PoorResponse
from quandoo.QuandooModel import urljoin, QuandooModel, QuandooDatetime


class Reservation(QuandooModel):

    def __init__(self, data, agent):
        if type(data) == dict:
            self.id = data["id"]
            self.number = data["number"]
            self.quandooId = data["quandooId"]
            self.status = data["status"]
            self.date = QuandooDatetime.parse_str_qdt(data["startTime"])
            self.startTime = QuandooDatetime.parse_str_qdt(data["startTime"])
            self.endTime = QuandooDatetime.parse_str_qdt(data["endTime"])
            self.capacity = data["capacity"]
            self.merchantId = data["merchantId"]
            self.customerId = data["customerId"]
            self.extraInfo = data["extraInfo"]
            self.createdAt = data["createdAt"]
            self.updatedAt = data["updatedAt"]

        else:
            self.id = data

        self.agent = agent

        super().__init__(data)

    def __str__(self):
        useful_attrs = []
        for key, val in self.__dict__.items():
            if key not in self.useless_attrs:
                if type(val) == QuandooDatetime:
                    if key == "date":
                        val = val.pretty_date().split(", ")[-1]
                    else:
                        val = val.pretty_date().split(", ")[0]
                useful_attrs.append("{}: {}".format(key, val))

        return "{}(\n\t{}\n)".format(
            self.__class__.__name__,
            ",\n\t".join(useful_attrs)
        )

    def _update(self, new_status: str=None, new_capacity: int=None, new_area_id: int=None, new_start_time: QuandooDatetime=None, reservation_tags=[]):
        data = {
            "reservation": {}
        }

        if new_status is not None:
            data["reservation"]["status"] = new_status
        if new_capacity is not None:
            data["reservation"]["capacity"] = new_capacity
        if new_area_id is not None:
            data["reservation"]["areaId"] = new_area_id
        if new_start_time is not None:
            data["reservation"]["dateTime"] = new_start_time.get_qdt()
        if reservation_tags:
            data["reservation"]['reservationTags'] = reservation_tags

        request = urljoin(self.agent.url, "reservations", self.id)
        response = requests.patch(request, headers=self.agent.headers, json=data)

        if response.status_code == 200:
            # TO DO
            # Change instance variables - by new fetch or local change?
            # new fetch is slower vs local change needs to re calc endTime
            return

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def cancel(self):
        self._update(new_status="CUSTOMER_CANCELED")
        self.status = "CUSTOMER_CANCELED"

    def reconfirm(self):
        self._update(new_status="RECONFIRMED")
        self.status = "RECONFIRMED"

    def change_capacity(self, new_capacity):
        self._update(new_capacity=new_capacity)
        self.capacity = new_capacity


class NewReservation(QuandooModel):

    def __init__(self, data, agent):
        self.id = data["reservation"]["id"]
        self.number = data["reservation"]["number"]
        self.status = data["reservation"]["status"]
        self.customerId = data["customer"]["id"]

        self.agent = agent

        super().__init__(data)

    def get_reservation(self):
        return self.agent.get_reservation(self.id)




import json

import requests

from quandoo.Error import PoorResponse
from quandoo.QuandooModel import PrettyClass, QuandooModel, QuandooDatetime, urljoin


class ReservationEnquiry(QuandooModel):

    def __init__(self, data, agent):
        self.id = data["id"]
        self.merchantId = data["merchantId"]
        self.customerId = data["customerId"]
        self.capacity = data["capacity"]
        self.startTime = data["startDateTime"]
        self.endTime = data["endDateTime"]
        self.status = data["status"]

        self.agent = agent

        super().__init__(data)

    def __change_status(self, new_status):
        data = {
            "status": new_status
        }
        request = urljoin(self.agent.url, "reservation-enquiries", self.id)
        response = requests.patch(request, headers=self.agent.headers, json=data)

        if response.status_code == 200:
            self.status = new_status
            return

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def get_messages(self):
        request = urljoin(self.agent.url, "reservation-enquiries", self.id, "messages")
        response = requests.get(request)

        if response.status_code == 200:
            return [Message(i) for i in json.loads(response.text)["messages"]]

        raise PoorResponse(response.status_code, json.loads(response.text), request)


class NewReservationEnquiry(QuandooModel):

    def __init__(self, data, agent):
        self.id = data["reservationEnquiry"]["id"]
        self.customerId = data["customer"]["id"]

        self.agent = agent

        super().__init__(data)

    def get_reservation_enquiry(self):
        return self.agent.get_reservation_enquiry(self.id)


class Message(PrettyClass):

    def __init__(self, data):
        self.senderType = data["senderType"]
        self.message = data["message"]
        self.creationDate = QuandooDatetime.parse_str_qdt(data["creationDate"])



import json
from datetime import timedelta

import requests

from quandoo.Customer import Customer
from quandoo.Error import PoorResponse
from quandoo.QuandooModel import QuandooModel, QuandooDatetime, urljoin
from quandoo.Reservation import Reservation, NewReservation
from quandoo.ReservationEnquiry import NewReservationEnquiry


class Merchant(QuandooModel):

    def __init__(self, data, agent):
        if type(data) == dict:
            self.id = data["id"]
            self.name = data["name"]
            address_vals = [i if i in data["location"]["address"].keys() else "" for i in ['number', 'street', 'city', 'country']]
            address_vals[1] = " ".join(address_vals[:1])
            address_vals = address_vals[1:]
            self.address = ", ".join(address_vals)

        else:
            self.id = data

        self.agent = agent

        super().__init__(data)

    def get_customers(self, offset=0, limit=100, modified_since: QuandooDatetime=None, modified_until: QuandooDatetime=None):
        params = {
            "offset": offset,
            "limit": limit
        }
        if modified_since is not None:
            params["modifiedSince"] = modified_since.get_urldt()
        if modified_until is not None:
            params["modifiedUntil"] = modified_until.get_urldt()

        request = urljoin(self.agent.url, "merchants", self.id, "customer")
        response = requests.get(request, headers=self.agent.headers, params=params)

        if response.status_code == 200:
            return [Customer(i, self.agent) for i in json.loads(response.text)["result"]]
        print(request)
        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def get_reservations(self, offset=0, limit=100, earliest=None, latest=None):
        params = {
            "offset": offset,
            "limit": limit
        }
        if earliest is not None:
            params["earliest"] = earliest.get_urldt()
        if latest is not None:
            params["latest"] = latest.get_urldt()

        request = urljoin(self.agent.url, "merchants", self.id, "reservations")
        response = requests.get(request, headers=self.agent.headers, params=params)

        if response.status_code == 200:
            return [Reservation(i, self.agent) for i in json.loads(response.text)["reservations"]]

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def get_available_times(self, pax: int, qdt: QuandooDatetime, duration=2, area_id=None):
        params = {
            "agentId": self.agent.agent_id,
            "capacity": pax,
            "fromTime": qdt.datetime.strftime("%H:%M"),
            "toTime": (qdt.datetime + timedelta(hours=duration)).strftime("%H:%M")
        }
        if area_id is not None:
            params["areaId"] = area_id

        request = urljoin(self.agent.url, "merchants", self.id, "availabilities", qdt.datetime.strftime("%Y-%m-%d"), "times")
        response = requests.get(request, headers=self.agent.headers, params=params)

        if response.status_code == 200:
            return [QuandooDatetime.parse_str_qdt(i["dateTime"]) for i in json.loads(response.text)["timeSlots"]]

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def is_available(self, pax: int, qdt: QuandooDatetime, duration=2, area_id=None):
        return qdt in self.get_available_times(pax, qdt, duration, area_id)

    def get_reviews(self, offset=0, limit=10):
        params = {
            "offset": offset,
            "limit": limit
        }

        request = urljoin(self.agent.url, "merchants", self.id, "reviews")
        response = requests.get(request, headers=self.agent.headers, params=params)

        if response.status_code == 200:
            return json.dumps(json.loads(response.text), indent=4)

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def create_reservation(self, customer, pax: int, qdt: QuandooDatetime, area_id=None, order_id=None, extra_info=None, reservation_tags=[]):
        data = {
            "reservation": {
                "merchantId": self.id,
                "capacity": pax,
                "dateTime": qdt.get_qdt()
            },
            "customer": customer.to_json(),
            "tracking": {
                "agent": {
                    "id": self.agent.agent_id
                }
            }
        }
        if area_id is not None:
            data["reservation"]["areaId"] = area_id
        if order_id is not None:
            data["reservation"]["orderId"] = order_id
        if extra_info is not None:
            data["reservation"]["extraInfo"] = extra_info
        if reservation_tags:
            data["reservation"]['reservationTags'] = reservation_tags

        request = urljoin(self.agent.url, "reservations")
        response = requests.put(request, headers=self.agent.headers, json=data)

        if response.status_code == 200:
            return NewReservation(json.loads(response.text), self.agent)

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def create_reservation_enquiry(self, customer, pax: int, start_qdt: QuandooDatetime, end_qdt: QuandooDatetime, message: str):
        data = {
            "reservationEnquiry": {
                "merchantId": self.id,
                "capacity": pax,
                "startDateTime": start_qdt.get_qdt(),
                "endDateTime": end_qdt.get_qdt(),
                "message": message
            },
            "customer": customer.to_json(),
            "tracking": {
                "agent": {
                    "id": self.agent.agent_id
                }
            }
        }

        request = urljoin(self.agent.url, "reservation-enquiries")
        response = requests.put(request, headers=self.agent.headers, json=data)

        if response.status_code == 201:
            return NewReservationEnquiry(json.loads(response.text), self.agent)

        raise PoorResponse(response.status_code, json.loads(response.text), request)

    def get_reservation_tags(self):
        request = urljoin(self.agent.url, 'merchants', self.id, 'reservation_tags')
        response = requests.put(request, headers=self.agent.headers)

        if response.status_code == 200:
            return json.dumps(json.loads(response.text), indent=4)

        raise PoorResponse(response.status_code, json.loads(response.text), request
        )



import unittest
import quandoo


class MyTestCase(unittest.TestCase):
    def test_prod_api(self):
        self.assertEqual(quandoo.status(), 200)

    def test_dev_api(self):
        self.assertEqual(quandoo.status_test(), 200)


if __name__ == '__main__':
    unittest.main()
